from dataclasses import dataclass, field
from typing import Any

from .errors import ConfigError

__all__ = ["GlobalConfig"]


@dataclass
class GlobalConfig:
    image_registry: str
    image_registry_api_url: str
    image_registry_api_token_url: str
    image_repository: str
    default_image_base: str
    agents: dict[str, dict[str, Any]] = field(default_factory=dict)

    @classmethod
    def from_mapping(cls, data: dict[str, Any]) -> "GlobalConfig":
        required = (
            "image_registry",
            "image_registry_api_url",
            "image_registry_api_token_url",
            "image_repository",
            "default_image_base",
        )
        missing = [key for key in required if key not in data]
        if missing:
            raise ConfigError(f"Missing required config values: {', '.join(missing)}.")
        return cls(
            image_registry=data["image_registry"],
            image_registry_api_url=data["image_registry_api_url"],
            image_registry_api_token_url=data["image_registry_api_token_url"],
            image_repository=data["image_repository"],
            default_image_base=data["default_image_base"],
            agents=data.get("agents", {}) or {},
        )

    def to_mapping(self) -> dict[str, Any]:
        return {
            "image_registry": self.image_registry,
            "image_registry_api_url": self.image_registry_api_url,
            "image_registry_api_token_url": self.image_registry_api_token_url,
            "image_repository": self.image_repository,
            "default_image_base": self.default_image_base,
            "agents": self.agents,
        }



from __future__ import annotations

import subprocess
import sys
from dataclasses import dataclass

from aicage.config.runtime_config import RunConfig
from aicage.errors import CliError
from aicage.registry import _local_query, _remote_query

__all__ = ["pull_image"]


@dataclass(frozen=True)
class _PullDecision:
    should_pull: bool


def pull_image(run_config: RunConfig) -> None:
    decision = _decide_pull(run_config)
    if not decision.should_pull:
        return

    _run_pull(run_config.image_ref)


def _decide_pull(run_config: RunConfig) -> _PullDecision:
    local_digest = _local_query.get_local_repo_digest(run_config)
    if local_digest is None:
        return _PullDecision(should_pull=True)

    remote_digest = _remote_query.get_remote_repo_digest(run_config)
    if remote_digest is None:
        return _PullDecision(should_pull=False)

    return _PullDecision(should_pull=local_digest != remote_digest)


def _run_pull(image_ref: str) -> None:
    print(f"[aicage] Pulling image {image_ref}...")

    last_nonempty_line = ""
    pull_process = subprocess.Popen(
        ["docker", "pull", image_ref],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        bufsize=1,
    )
    if pull_process.stdout is not None:
        for line in pull_process.stdout:
            sys.stdout.write(line)
            sys.stdout.flush()
            stripped = line.strip()
            if stripped:
                last_nonempty_line = stripped

    pull_process.wait()

    if pull_process.returncode == 0:
        return

    inspect = subprocess.run(
        ["docker", "image", "inspect", image_ref],
        check=False,
        capture_output=True,
        text=True,
    )
    if inspect.returncode == 0:
        msg = last_nonempty_line or f"docker pull failed for {image_ref}"
        print(f"[aicage] Warning: {msg}. Using local image.", file=sys.stderr)
        return

    detail = last_nonempty_line or f"docker pull failed for {image_ref}"
    raise CliError(detail)



from __future__ import annotations

import importlib
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from aicage.config.context import ConfigContext
    from aicage.config.runtime_config import RunConfig

__all__ = ["pull_image", "select_agent_image"]


def pull_image(run_config: RunConfig) -> None:
    module = importlib.import_module("aicage.registry.image_pull")
    module.pull_image(run_config)


def select_agent_image(agent: str, context: ConfigContext) -> str:
    module = importlib.import_module("aicage.registry.image_selection")
    return module.select_agent_image(agent, context)



from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

__all__ = ["ProjectConfig", "AgentConfig", "AgentMounts"]


@dataclass
class AgentMounts:
    gitconfig: bool | None = None
    gnupg: bool | None = None
    ssh: bool | None = None
    docker: bool | None = None

    @classmethod
    def from_mapping(cls, data: dict[str, Any]) -> "AgentMounts":
        return cls(
            gitconfig=data.get("gitconfig"),
            gnupg=data.get("gnupg"),
            ssh=data.get("ssh"),
            docker=data.get("docker"),
        )

    def to_mapping(self) -> dict[str, bool]:
        payload: dict[str, bool] = {}
        if self.gitconfig is not None:
            payload["gitconfig"] = self.gitconfig
        if self.gnupg is not None:
            payload["gnupg"] = self.gnupg
        if self.ssh is not None:
            payload["ssh"] = self.ssh
        if self.docker is not None:
            payload["docker"] = self.docker
        return payload


@dataclass
class AgentConfig:
    base: str | None = None
    docker_args: str = ""
    entrypoint: str | None = None
    mounts: AgentMounts = field(default_factory=AgentMounts)

    @classmethod
    def from_mapping(cls, data: dict[str, Any]) -> "AgentConfig":
        mounts = AgentMounts.from_mapping(data.get("mounts", {}) or {})
        return cls(
            base=data.get("base"),
            docker_args=data.get("docker_args", "") or "",
            entrypoint=data.get("entrypoint"),
            mounts=mounts,
        )

    def to_mapping(self) -> dict[str, Any]:
        payload: dict[str, Any] = {}
        if self.base:
            payload["base"] = self.base
        if self.docker_args:
            payload["docker_args"] = self.docker_args
        if self.entrypoint:
            payload["entrypoint"] = self.entrypoint
        mounts = self.mounts.to_mapping()
        if mounts:
            payload["mounts"] = mounts
        return payload


@dataclass
class ProjectConfig:
    path: str
    agents: dict[str, AgentConfig] = field(default_factory=dict)

    @classmethod
    def from_mapping(cls, project_path: Path, data: dict[str, Any]) -> "ProjectConfig":
        raw_agents = data.get("agents", {}) or {}
        agents = {name: AgentConfig.from_mapping(cfg) for name, cfg in raw_agents.items()}
        legacy_docker_args = data.get("docker_args", "")
        if legacy_docker_args:
            for agent_cfg in agents.values():
                if not agent_cfg.docker_args:
                    agent_cfg.docker_args = legacy_docker_args
        return cls(
            path=data.get("path", str(project_path)),
            agents=agents,
        )

    def to_mapping(self) -> dict[str, Any]:
        agents_payload = {name: cfg.to_mapping() for name, cfg in self.agents.items()}
        return {"path": self.path, "agents": agents_payload}



from .config_store import SettingsStore
from .context import ConfigContext, build_config_context
from .errors import ConfigError
from .global_config import GlobalConfig
from .project_config import ProjectConfig
from .runtime_config import RunConfig, load_run_config

__all__ = [
    "ConfigContext",
    "ConfigError",
    "SettingsStore",
    "GlobalConfig",
    "ProjectConfig",
    "RunConfig",
    "build_config_context",
    "load_run_config",
]



import shlex
import subprocess
import sys
from collections.abc import Sequence
from pathlib import Path

from aicage.cli_parse import parse_cli
from aicage.cli_types import ParsedArgs
from aicage.config import ConfigError, RunConfig, SettingsStore, load_run_config
from aicage.errors import CliError
from aicage.registry import pull_image
from aicage.runtime.run_args import DockerRunArgs, assemble_docker_run
from aicage.runtime.run_plan import build_run_args

__all__ = ["ParsedArgs", "parse_cli", "main"]


def _print_project_config() -> None:
    store = SettingsStore()
    project_path = Path.cwd().resolve()
    config_path = store.project_config_path(project_path)
    print("Project config path:")
    print(config_path)
    print()
    print("Project config content:")
    if config_path.exists():
        contents = config_path.read_text(encoding="utf-8").rstrip()
        if contents:
            print(contents)
        else:
            print("(empty)")
    else:
        print("(missing)")


def main(argv: Sequence[str] | None = None) -> int:
    parsed_argv: Sequence[str] = argv if argv is not None else sys.argv[1:]
    try:
        parsed: ParsedArgs = parse_cli(parsed_argv)
        if parsed.config_action == "print":
            _print_project_config()
            return 0
        run_config: RunConfig = load_run_config(parsed.agent, parsed)
        pull_image(run_config)
        run_args: DockerRunArgs = build_run_args(config=run_config, parsed=parsed)

        run_cmd: list[str] = assemble_docker_run(run_args)

        if parsed.dry_run:
            print(shlex.join(run_cmd))
            return 0

        subprocess.run(run_cmd, check=True)
        return 0
    except KeyboardInterrupt:
        print()
        return 130
    except (CliError, ConfigError) as exc:
        print(f"[aicage] {exc}", file=sys.stderr)
        return 1


if __name__ == "__main__":
    sys.exit(main())



from __future__ import annotations

from aicage.config.resources import find_packaged_path
from aicage.errors import CliError

from .models import ImagesMetadata

__all__ = ["load_images_metadata"]


def load_images_metadata() -> ImagesMetadata:
    path = find_packaged_path("images-metadata.yaml")
    try:
        payload = path.read_text(encoding="utf-8")
    except OSError as exc:
        raise CliError(f"Failed to read images metadata from {path}: {exc}") from exc
    return ImagesMetadata.from_yaml(payload)
